<?php

namespace BS\ChatGPTQuickReplyBot\Job;

use BS\ChatGPTQuickReplyBot\Listener;
use XF;
use XF\Entity\Post;
use XF\Entity\Thread;
use XF\Entity\User;
use XF\Job\AbstractJob;

class AnswerOnQuote extends AbstractJob
{
    public function run($maxRunTime)
    {
        $em = XF::em();
        $thread = $em->find('XF:Thread', $this->data['thread_id'], ['Forum']);
        /** @var \XF\Entity\Post $post */
        $post = $em->find('XF:Post', $this->data['post_id']);
        $botUser = Listener::getBot();

        if (! $post || ! $thread || ! $botUser) {
            return $this->complete();
        }

        $quotes = $this->getQuotes($post, $botUser);
        if (empty($quotes)) {
            return $this->complete();
        }

        if (! $this->shouldReply($thread, $post, $botUser)) {
            return $this->complete();
        }

        /** @var \BS\ChatGPTQuickReplyBot\Service\ReplyOnPost $replyOnPost */
        $replyOnPost = XF::service('BS\ChatGPTQuickReplyBot:ReplyOnPost', $thread, $botUser);
        $replyOnPost->getRepliesOnQuotes($post, $quotes);

        XF::asVisitor($botUser, static function () use ($replyOnPost) {
            $replyOnPost->postReplies();
        });

        return $this->complete();
    }

    protected function getQuotes(Post $post, User $bot): array
    {
        /** @var \BS\ChatGPTBots\Repository\Message $messageRepo */
        $messageRepo = XF::repository('BS\ChatGPTBots:Message');
        return $messageRepo->getQuotes(
            $post->message,
            $bot->user_id
        );
    }

    protected function shouldReply(Thread $thread, Post $post, User $bot): bool
    {
        $options = XF::options();

        if ($options->bsChatGPTAutoresponderAnswerToTSOnly
            && $post->user_id !== $thread->user_id
        ) {
            return false;
        }

        $maxMessagesPerThread = (int)$options->bsChatGPTAutoresponderMaxMessagesPerThread;
        $maxMessagesPerThreadForForum = $post->User->hasNodePermission(
            $thread->node_id,
            'CGPTA_maxRepliesPerThread'
        );
        if (! ($maxMessagesPerThread || $maxMessagesPerThreadForForum)) {
            return true;
        }

        /** @var \XF\Finder\Post $postFinder */
        $postFinder = XF::finder('XF:Post')
            ->inThread($thread, ['visibility' => 'visible'])
            ->where('user_id', $bot->user_id);
        $postsCount = $postFinder->total();

        if ($maxMessagesPerThreadForForum > 0
            && $postsCount >= $maxMessagesPerThreadForForum
        ) {
           return false;
        }

        return !$maxMessagesPerThread || $postsCount < $maxMessagesPerThread;
    }

    public function getStatusMessage()
    {
        return 'Answering questions...';
    }

    public function canCancel()
    {
        return false;
    }

    public function canTriggerByChoice()
    {
        return false;
    }
}