<?php

namespace BS\ChatGPTQuickReplyBot;

use XF\Entity\Post;
use XF\Entity\Thread;
use XF\Entity\User;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Manager;
use XF\Mvc\Entity\Structure;

class Listener
{
    public static function entityStructureThread(Manager $em, Structure $structure)
    {
        $structure->columns['bs_chagpt_autoresponder_enable'] = [
            'type' => Entity::BOOL,
            'default' => \XF::options()->bsChatGPTAutoresponderInThreadDefault,
        ];
    }

    public static function threadPostSave(Thread $thread)
    {
        /** @var \BS\ChatGPTQuickReplyBot\Service\ReplyJobRunner $jobRunner */
        $jobRunner = \XF::service('BS\ChatGPTQuickReplyBot:ReplyJobRunner');
        if (! $jobRunner->shouldRunJobForThread($thread)) {
            return;
        }

        $jobRunner->runJobForThread($thread);
    }

    public static function postPostSave(Post $post)
    {
        /** @var \BS\ChatGPTQuickReplyBot\Service\ReplyJobRunner $jobRunner */
        $jobRunner = \XF::service('BS\ChatGPTQuickReplyBot:ReplyJobRunner');
        if (! $jobRunner->shouldRunJobForPost($post)) {
            return;
        }

        $jobRunner->runJobForPost($post);
    }

    public static function getBot(): ?User
    {
        $app = \XF::app();
        $options = $app->options();
        $botUser = $app->em()->findOne('XF:User', ['username' => $options->bsChatGPTQuickReplyUser]);
        if (! $botUser) {
            return null;
        }

        return $botUser;
    }
}