<?php

namespace BS\ChatGPTQuickReplyBot\Service;

use BS\ChatGPTBots\Repository\Message;
use BS\ChatGPTBots\Response;
use BS\ChatGPTQuickReplyBot\Support\BotHelper;
use Orhanerday\OpenAi\OpenAi;
use XF\App;
use XF\Entity\Thread;
use XF\Entity\User;
use XF\Service\AbstractService;

class QuickReply extends AbstractService
{
    protected User $user;
    protected Thread $thread;
    protected OpenAi $api;
    private Message $messageRepo;
    public function __construct(
        App $app,
        User $user,
        Thread $thread
    ) {
        parent::__construct($app);
        $this->setUser($user);
        $this->setThread($thread);
        $this->api = $app->container('chatGPT');
        $this->messageRepo = $this->repository('BS\ChatGPTBots:Message');
    }

    public function answer()
    {
        $options = \XF::options();
        $reply = Response::getReplyWithLogErrors($this->api, [
            'model'             => $options->bsChatGPTAutoresponderModel,
            'messages'          => $this->buildMessages(),
            'temperature'       => (float)$options->bsChatGPTQuickReplyTemp,
            'frequency_penalty' => 0,
            'presence_penalty'  => 0,
        ]);

        \XF::asVisitor($this->user, function () use ($reply) {
            $this->postReply($reply);
        });
    }

    protected function postReply(string $message)
    {
        /** @var \XF\Service\Thread\Replier $replier */
        $replier = $this->service('XF:Thread\Replier', $this->thread);
        $replier->setMessage($message);
        $replier->setIsAutomated();
        $replier->save();
        $replier->sendNotifications();
    }

    protected function buildMessages()
    {
        return [
            BotHelper::getSystemForThreadReply($this->thread),
            $this->messageRepo->wrapMessage($this->thread->FirstPost->message)
        ];
    }

    /**
     * @param  \XF\Entity\User  $user
     */
    public function setUser(User $user): void
    {
        $this->user = $user;
    }

    /**
     * @param  \XF\Entity\Thread  $thread
     */
    public function setThread(Thread $thread): void
    {
        $this->thread = $thread;
    }
}