<?php

namespace BS\ChatGPTQuickReplyBot\Service;

use XF\Entity\Post;
use XF\Entity\Thread;
use XF\Service\AbstractService;
class ReplyJobRunner extends AbstractService
{
    public function shouldRunJobForThread(Thread $thread): bool
    {
        if (! $this->app->container('chatGPT')) {
            return false;
        }

        // Reply only if thread is visible
        if (! $thread->isVisible()) {
            return false;
        }

        // Reply only if thread is new (include threads after moderation)
        $insertingOrLeaveFromModeration = $thread->isInsert()
            || $thread->isStateChanged('discussion_state', 'moderated') === 'leave';
        if (! $insertingOrLeaveFromModeration) {
            return false;
        }

        $user = $thread->User;
        if (! $user) {
            return false;
        }

        if (! $user->hasNodePermission($thread->node_id, 'useChatGptAutoresponder')) {
            return false;
        }

        if (! $thread->bs_chagpt_autoresponder_enable) {
            return false;
        }

        if (! \XF::service('BS\ChatGPTQuickReplyBot:ChatGPT')->isThreadAnswered($thread)) {
            return false;
        }

        $options = $this->app->options();
        $quickReplyNodeIds = $options->bsChatGPTQuickReplyNodes;
        return in_array($thread->node_id, $quickReplyNodeIds);
    }

    public function shouldRunJobForPost(Post $post): bool
    {
        if (! $this->app->container('chatGPT')) {
            return false;
        }

        // Reply only if post is visible
        if (! $post->isVisible()) {
            return false;
        }

        $user = $post->User;
        if (! $user) {
            return false;
        }

        if (! $user->hasNodePermission($post->Thread->node_id, 'useChatGptAutoresponder')) {
            return false;
        }

        // Reply only if post is new (include posts after moderation)
        $insertingOrLeaveFromModeration = $post->isInsert()
            || $post->isStateChanged('message_state', 'moderated') === 'leave';
        if (! $insertingOrLeaveFromModeration) {
            return false;
        }

        // Check is AI answers enabled for thread
        $thread = $post->Thread;
        if (! $thread->bs_chagpt_autoresponder_enable) {
            return false;
        }

        $options = $this->app->options();

        if ($post->username === $options->bsChatGPTQuickReplyUser) {
            return false;
        }

        $quickReplyNodeIds = $options->bsChatGPTQuickReplyNodes;
        return $thread->isVisible()
            && in_array($thread->node_id, $quickReplyNodeIds);
    }

    public function runJobForThread(Thread $thread)
    {
        $this->app->jobManager()->enqueue(
            'BS\ChatGPTQuickReplyBot:Answer',
            ['thread_id' => $thread->thread_id],
            false
        );
    }

    public function runJobForPost(Post $post)
    {
        $this->app->jobManager()->enqueue(
            'BS\ChatGPTQuickReplyBot:AnswerOnQuote',
            ['thread_id' => $post->Thread->thread_id, 'post_id' => $post->post_id],
            false
        );
    }
}