<?php

namespace BS\ChatGPTQuickReplyBot;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

class Setup extends AbstractSetup
{
    use StepRunnerInstallTrait;
    use StepRunnerUpgradeTrait;
    use StepRunnerUninstallTrait;

    public function installStep1()
    {
        $this->addAutoresponderEnableToThreadTable();
    }

    public function upgrade1020070Step1()
    {
        $this->addAutoresponderEnableToThreadTable();
    }

    public function checkRequirements(&$errors = [], &$warnings = [])
    {
        /** @var \BS\ChatGPTQuickReplyBot\Service\LicenseChecker $checker */
        $checker = \XF::service('BS\ChatGPTQuickReplyBot:LicenseChecker');
        if (! $checker->isAllowedAction('install')) {
            $errors[] = 'Please purchase a license for this add-on before installing it.';
        }

        $this->clearOsRequirementsCache();
    }

    public function postUpgrade($previousVersion, array &$stateChanges)
    {
        $actionData = [
            'previous_version' => $previousVersion,
            'current_version' => $this->addOn->getJsonVersion()
        ];
        $this->assertLicenceAllowed('upgrade', $actionData);
    }

    public function postRebuild()
    {
        $this->assertLicenceAllowed('rebuild');

        $this->clearOsRequirementsCache();
    }

    protected function assertLicenceAllowed(string $action, array $actionData = []): void
    {
        /** @var \BS\ChatGPTQuickReplyBot\Service\LicenseChecker $checker */
        $checker = \XF::service('BS\ChatGPTQuickReplyBot:LicenseChecker');
        if (! $checker->isAllowedAction($action, $actionData)) {
            \XF::registry()->set('bsCQRBLicense', false);
        } else {
            \XF::registry()->offsetUnset('bsCQRBLicense');
        }
    }

    public function uninstallStep1()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_thread', function (Alter $table) {
            $table->dropColumns(['bs_chagpt_autoresponder_enable']);
        });
    }

    protected function addAutoresponderEnableToThreadTable()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_thread', function (Alter $table) {
            $table->addColumn('bs_chagpt_autoresponder_enable', 'tinyint')->setDefault(1);
        });
    }

    protected function clearOsRequirementsCache()
    {
        \XF::registry()->offsetUnset('bsCQRBRequirementsFn');
    }
}